#!/bin/sh
# Copyright(c) 2007 Red Hat Middleware, LLC,
# and individual contributors as indicated by the @authors tag.
# See the copyright.txt in the distribution for a
# full listing of individual contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library in the file COPYING.LIB;
# if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# @author Jean-Frederic Clere
# @author Mladen Turk
#
echo ""
echo "Running `basename $0` $LastChangedDate: 2007-07-24 08:37:26 -0400 (Tue, 24 Jul 2007) $"
echo ""
echo "Started : `date`"
echo "Params  : $@"
echo ""

# Globals
build_top=`pwd`
NATIVEEOL=LF
build_version=""
PACKAGE=""
BUILD_WIN=false

package_list=${build_top}/package.list
awk=awk
has_posix_tar=true

# parameters
has_openssl=false
has_cache=false
has_version=false
has_package=false
has_static=false

while [ "x" != "x$1" ]
do
  case  $1 in
    -ssl)
      has_openssl=true
      ;;
    -cache)
      has_cache=true
      ;;
    -static)
      has_static=true
      ;;
    [0-9]*)
      build_version=`echo $1`
      has_version=true
      ;;
    [a-z]*)
      PACKAGE=`echo $1`
      has_package=true
      ;;
    *)
      echo "Parameter $1 not supported"
      exit 1
      ;;
  esac
  shift
done

# Check if we just have only to build the binaries
# something like package-version-src or
# package-version-src-ssl (where package is jboss-blalbla).
if [ -d srclib ]; then
  package_list=${build_top}/package.list
  PACKAGE=`basename ${build_top} | awk -F - '{ print $1 "-" $2 }'`
  has_package=true
  has_version=true
  TYPE=`basename ${build_top} | awk -F - '{ print $NF }'`
  if [ $TYPE = "src" ]; then
    build_version=`basename ${build_top} | awk -F - '{ print $(NF-1) }'`
  else
    build_version=`basename ${build_top} | awk -F - '{ print $(NF-2) }'`
  fi
  echo "build_version : $build_version"
  echo "PACKAGE       : $PACKAGE"
fi

# check parameters.
if [ $has_version -a ! $has_package ]; then
  echo "Can't use version without package"
  exit 1
fi

# package.
if $has_package; then
  grep "^$PACKAGE" $package_list >/dev/null
  if [ $? -ne 0 ]; then
    echo "Canot find $PACKAGE in `basename ${package_list}`"
    exit 1
  fi
else
  echo "`basename $0`: Missing package name"
  echo "Usage `basename $0` package-name [options]."
  echo ""
  exit 1
fi

# get lastest version if needed.
if $has_version; then
  echo "Checking $PACKAGE version $PACKAGE from `basename ${package_list}`"
  grep "^$PACKAGE" ${package_list} | grep ${build_version} >/dev/null
  if [ $? -ne 0 ]; then
    echo "Canot find $PACKAGE $build_version in `basename ${package_list}`"
    exit 1
  fi
else
  parg=`grep -v '^#' ${package_list} | grep "^$PACKAGE|" | tail -1`
  build_version=`echo "${parg}" | ${awk} -F'|' '{print $2}'`
  has_version=true
  echo "Using version ${build_version} from `basename ${package_list}`"
fi

# Read apr and openssl version.
parg=`grep -v '^#' ${package_list} | grep "^$PACKAGE|" | grep $build_version`
apr_version=`echo "${parg}" | ${awk} -F'|' '{print $3}'`
apu_version=`echo "${parg}" | ${awk} -F'|' '{print $4}'`
ssl_version=`echo "${parg}" | ${awk} -F'|' '{print $5}'`
svn_tagname=`echo "${parg}" | ${awk} -F'|' '{print $6}'`

if [  "x" = "x${ssl_version}" ]; then
  has_openssl=false
fi

# Get plaform information
so_extension=so
if [  "x" = "x$BUILD_CPU" ]; then
  BUILD_CPU=`uname -m`
fi
case ${BUILD_CPU} in
  sun4u*)
    BUILD_CPU=sparcv9
  ;;
  i86pc*)
    BUILD_CPU=i686
  ;;
  x86_64*)
    BUILD_CPU=amd64
  ;;
  9000/800*)
    BUILD_CPU=parisc2
    so_extension=sl
  ;;
  Power*)
    BUILD_CPU=ppc
    # Add the default java location
    if [ "x${JAVA_HOME}" = "x" ]; then
      JAVA_HOME=/System/Library/Frameworks/JavaVM.framework/Versions/CurrentJDK/
      export JAVA_HOME
    fi
  ;;
esac

if [  "x" = "x$BUILD_SYS" ]; then
  BUILD_SYS=`uname -s`
fi
case ${BUILD_SYS} in
    Linux*)
    BUILD_SYS="linux2"
    ;;
    SunOS*)
    BUILD_SYS="solaris"
    # Try to use sun studio.
    # we need makedepend for make depend (openssl)
    PATH=/opt/SUNWspro/bin:/usr/X/bin:/usr/local/bin:$PATH
    export PATH
    ;;
    HP-UX*)
    BUILD_SYS="hpux"
    has_posix_tar=false
    ;;
    windows)
    so_extension=dll
    BUILD_WIN=true
    NATIVEEOL="CRLF"
    ;;
    CYGWIN*)
    so_extension=dll
    BUILD_WIN=true
    NATIVEEOL="CRLF"
    BUILD_SYS=windows
    ;;
esac
export so_extension
export NATIVEEOL
export BUILD_SYS
export BUILD_CPU
export BUILD_WIN
BUILD_TAG=${BUILD_SYS}-${BUILD_CPU}
export BUILD_TAG

# hack use our utilities instead the broken system ones.
if [ -d /home/shared/$BUILD_SYS-$BUILD_CPU/bin ]; then
  PATH=/home/shared/$BUILD_SYS-$BUILD_CPU/bin:$PATH
  export PATH
  echo "PATH changed to $PATH"
fi

build_cache_dir=${build_top}/cache
build_working_dir=${build_top}/work
build_output_dir=${build_top}/output

mkdir -p ${build_working_dir}
mkdir -p ${build_output_dir}

build_common_dir=${build_cache_dir}/common/${BUILD_TAG}
if $has_cache ; then
  mkdir -p ${build_common_dir}
else
  rm -rf ${build_common_dir}
  mkdir -p ${build_common_dir}
fi

# download and copy the srclib stuff
package_src_name=${PACKAGE}-${build_version}-src
if $has_openssl; then
  package_src_name=${package_src_name}-ssl
fi
package_build_name=${PACKAGE}-${build_version}-${BUILD_SYS}-${BUILD_CPU}
if $has_openssl; then
  package_build_name=${package_build_name}-ssl
fi
package_src_dir=${build_working_dir}/${package_src_name}
package_dist_dir=${build_working_dir}/${package_src_name}/dist/${BUILD_TAG}
package_output_dir=${build_working_dir}/${package_src_name}/output/${BUILD_TAG}

# Override a shell command
# # $1=script $2..n
override()
{
  cmd=`basename $1 .sh`
  if [ -x  ${cmd}.${BUILD_SYS}.${BUILD_CPU}.sh ]; then
    run=${cmd}.${BUILD_SYS}.${BUILD_CPU}.sh
  elif [ -x ${cmd}.${BUILD_SYS}.sh ]; then
    run=${cmd}.${BUILD_SYS}.sh
  else
    run=${cmd}.sh
  fi
  shift
  ./$run $@
  if [ $? -eq 2 ]; then
    echo "$run not supported on this platform"
    exit 0
  fi
  if [ $? -eq 1 ]; then
    echo "$run failed"
    exit 1
  fi
}

# Download the sources. It guess how to do it.
# $1: URL.
downloadtaredsrc()
{
  URL=$1
  wget --tries=0 --retry-connrefused $URL
  fname=`basename $URL`
  case ${fname} in
    *.tar.gz)
      gunzip -c $fname | tar -xf -
      ;;
    *.zip)
      unzip -q -o $fname
      ;;
  esac
  rm -f $fname
}

# Wrap command line arguments to windows
# $1: Use back slashes
# $@: Arguments
wrapped_args=""
cygwinpath()
{
    back=$1
    shift

    # Convert the mountpoint in parameters to Win32 filenames
    # For instance: /cygdrive/c/foo -> c:/foo
    wrapped_args=`echo $1 | sed 's/\/cygdrive\/\(.\)\(.*\)/\1:\2/'`
    if ${back} ; then
        # Convert forward to back slashes
        # For instance: c:/foo - > c:\foo
        wrapped_args=`echo ${wrapped_args} | sed "s/\\//\\\\\\/g"`
    fi
    shift
    for i in "${@}"
    do
        i=`echo ${i} | sed 's/\/cygdrive\/\(.\)\(.*\)/\1:\2/'`
        if ${back} ; then
            # Convert forward to back slashes
            # For instance: c:/foo - > c:\foo
            i=`echo ${i} | sed "s/\\//\\\\\\/g"`
        fi
        wrapped_args="${wrapped_args} ${i}"
    done
}

#
# Build a tarball or a zip file
# $1: From where to take the files.
# $2: regexp with the files name (quoted string)
# $3: Where to put the tarball
# $4: Basename of the tarball
buildtar()
{
  src_dir=$1
  src_files=$2
  dst_dir=$3
  dst_name=$4
  if $BUILD_WIN ; then
    package_extension=zip
  else
    package_extension=tar.gz
  fi
  export package_extension

  # Remove any previous tarball
  rm -f ${dst_dir}/${dst_name}.$package_extension

  (cd ${src_dir}
  if $BUILD_WIN ; then
    zip -9rqo ${dst_dir}/${dst_name}.zip ${src_files}
  else
    if $has_posix_tar; then
      tar --owner=root --group=bin -cf - ${src_files} | gzip -c > ${dst_dir}/${dst_name}.tar.gz
    else
      tar -cf - * | gzip -c > ${dst_dir}/${dst_name}.tar.gz
    fi
  fi
  )
  ls -l ${dst_dir}/${dst_name}.$package_extension
}

#
# Build openssl
# $1 directory of the openssl sources
# $2 static or shared
buildopenssl()
{
  ssl_srcdir=$1
  ssl_static=$2
  if $ssl_static ; then
    ssl_common_dir=${build_common_dir}-static
  else
    ssl_common_dir=${build_common_dir}
  fi

  if [ -f ${ssl_common_dir}/include/openssl/opensslv.h ]; then
    echo "Using cached openssl in ${ssl_common_dir}"
    return
  fi
  SSLNUM=`basename ${ssl_srcdir} | awk -F - '{ print $2}' | sed 's:[a-z]::'`
  echo "Configuring OpenSSL ${ssl_version} for ${BUILD_TAG} ..."
  ssl_common_flags="threads no-zlib no-zlib-dynamic no-gmp no-krb5 no-rc5 no-mdc2 no-idea no-ec"
  if $ssl_static ; then
    ssl_build_flags="no-asm no-shared"
  else
    ssl_build_flags="shared"
  fi
  ssl_target="--prefix=${ssl_common_dir} --openssldir=${ssl_common_dir}/ssl"
  (cd ${ssl_srcdir}
   case ${BUILD_TAG} in
     solaris-sparcv9)
        ./Configure ${ssl_target} ${ssl_common_flags} ${ssl_build_flags} solaris-sparcv9-cc
      ;;
     solaris-i686)
        ./Configure ${ssl_target} ${ssl_common_flags} ${ssl_build_flags} solaris-x86-cc
      ;;
     hpux-parisc2)
        ./Configure ${ssl_target} ${ssl_common_flags} ${ssl_build_flags} hpux-parisc2-cc
      ;;
     hpux-ia64)
        ./Configure ${ssl_target} ${ssl_common_flags} ${ssl_build_flags} hpux-ia64-cc
      ;;
    windows-i686)
        ./Configure ${ssl_target} VC-NT
        ms/do_masm.bat
      ;;
    windows-amd64)
        ./Configure ${ssl_target} VC-WIN64A
        ms/do_win64a.bat
      ;;
    windows-ia64)
        ./Configure ${ssl_target} VC-WIN64I
        ms/do_win64i.bat
      ;;
     *)
        ./config ${ssl_target} -fPIC ${ssl_common_flags} ${ssl_build_flags}
      ;;
   esac
   echo "Building OpenSSL ${ssl_version} for ${BUILD_TAG} ..."
   if $BUILD_WIN ; then
     if $ssl_static ; then
        nmake -f ms/nt.mak install
     else
        nmake -f ms/nt.mak install
     fi
   else
     make depend
     make
     make install_sw
     if [ ! $ssl_static ] ; then
       (cd ${ssl_common_dir}/lib
        ln -s libcrypto.${so_extension}.${SSLNUM} libcrypto.${so_extension}.0
        ln -s libssl.${so_extension}.${SSLNUM} libssl.${so_extension}.0
       )
     fi
   fi
  )
}

#
# Build apr
# $1 directory of the sources directory
# $2 static or shared
buildapr()
{
  apr_srcdir=$1
  apr_static=$2

  if $apr_static ; then
    apr_common_dir=${build_common_dir}-static
  else
    apr_common_dir=${build_common_dir}
  fi

  if [ -f ${apr_common_dir}/include/apr-1/apr.h ]; then
    echo "Using cached apr in ${apr_common_dir}"
    return
  fi
  echo "Configuring apr-${apr_version} with --prefix=${apr_common_dir}"
  if $BUILD_WIN ; then
    (cd ${apr_srcdir}
      cp -f include/apr.hw include/apr.h
      echo "Building APR ${apr_version} ..."
      cygwinpath true PREFIX=${apr_common_dir} SRCDIR=`pwd`
      if $apr_static ; then
        nmake -f NMAKEmakefile $wrapped_args APR_DECLARE_STATIC=true install
      else
        nmake -f NMAKEmakefile $wrapped_args install
      fi
    )
  else
    (cd ${apr_srcdir}
      ./configure --prefix=${apr_common_dir}
      echo "Building APR ${apr_version} ..."
      make
      make install
    )
  fi
}

#
# Build apr-util
# $1 directory of the sources directory
# $2 static or shared
buildapu()
{
  apu_srcdir=$1
  apu_static=$2

  if $apu_static ; then
    apu_common_dir=${build_common_dir}-static
  else
    apu_common_dir=${build_common_dir}
  fi

  if [ -f ${apu_common_dir}/include/apr-1/apu.h ]; then
    echo "Using cached apr-util in ${apu_common_dir}"
    return
  fi
  echo "Configuring apr-util-${apu_version} with --prefix=${apu_common_dir}"
  if $BUILD_WIN ; then
    (cd ${apu_srcdir}
      cp -f include/apu.hw include/apu.h
      cp -f xml/expat/lib/winconfig.h xml/expat/lib/config.h
      cp -f xml/expat/lib/expat.h.in xml/expat/lib/expat.h
      echo "Building APR utilities ${apu_version} ..."
      cygwinpath true WITH_APR=${apu_common_dir} PREFIX=${apu_common_dir} SRCDIR=`pwd`
      if $apu_static ; then
        nmake -f NMAKEmakefile $wrapped_args APR_DECLARE_STATIC=true install
      else
        nmake -f NMAKEmakefile $wrapped_args install
      fi
    )
  else
    (cd ${apu_srcdir}
      ./configure --with-apr=${apu_common_dir} --prefix=${apu_common_dir} --with-expat=builtin --with-dbm=sdbm
      echo "Building APR utilities ${apu_version} ..."
      make
      make install
    )
  fi
}

#
# Build the binaries
# $1 directory where to put the resulting tarball.
#    XXX: It is always $build_output_dir !
buildbin()
{
  echo "Building binaries at `pwd`"
  dst_dir=$1

  buildapr srclib/apr-${apr_version} false
  if $has_static ; then
    buildapr srclib/apr-${apr_version} true
  fi
  buildapu srclib/apr-util-${apu_version} false
  if $has_static ; then
    buildapu srclib/apr-util-${apu_version} true
  fi

  ls srclib | grep openssl >/dev/null
  if [ $? -eq 0 ]; then
    buildopenssl srclib/openssl-${ssl_version} false
    if $has_static ; then
      buildopenssl srclib/openssl-${ssl_version} false
    fi
  fi
  # Do our specific part.
  override ${package_src_dir}/buildbin.${PACKAGE}.sh ${build_common_dir} ${package_dist_dir} ${package_output_dir} $has_openssl $has_static

  # General part
  cp JBossORG-EULA.txt ${package_output_dir}

  # Build the binary distribution tarball
  buildtar ${package_output_dir} "*" ${dst_dir} ${package_build_name}
}

# Only buid binaries?
if [ -d srclib ]; then
  buildbin ${build_output_dir}
  exit 0
fi

# Apply patch for the component
# $1 directory of the component like apr-1.2.8
applypatch()
{
  DIR=$1
  if [ -f $build_top/../patch/$DIR.patch ];
  then
    (cd $DIR
    patch -tfs -p0 -i $build_top/../patch/$DIR.patch
    )
  fi
}

# Check and download
# $1 directory of the checkout directory
# $2 url for the download
ckeckdownload()
{
  src_dir=$1
  src_url=$2
  cd $build_cache_dir
  if [ ! -d ${src_dir} ]; then
    downloadtaredsrc $src_url
    applypatch $src_dir
  fi
  cp -rp ${src_dir} ${package_src_dir}/srclib
  cd $build_top
}

# set the urls
if $BUILD_WIN ; then
  APRURL=http://ftp.heanet.ie/mirrors/www.apache.org/dist/apr/apr-${apr_version}-win32-src.zip
  APUURL=http://ftp.heanet.ie/mirrors/www.apache.org/dist/apr/apr-util-${apu_version}-win32-src.zip
else
  APRURL=http://ftp.heanet.ie/mirrors/www.apache.org/dist/apr/apr-${apr_version}.tar.gz
  APUURL=http://ftp.heanet.ie/mirrors/www.apache.org/dist/apr/apr-util-${apu_version}.tar.gz
fi
SSLURL=http://www.openssl.org/source/openssl-${ssl_version}.tar.gz

rm -rf ${package_src_dir}
mkdir -p ${package_src_dir}/srclib

ckeckdownload apr-${apr_version} $APRURL
ckeckdownload apr-util-${apu_version} $APUURL
if $has_openssl; then
  ckeckdownload openssl-${ssl_version} $SSLURL
fi

#
# Get the repository of what we build.
override ${build_top}/buildsrc.${PACKAGE}.sh ${svn_tagname} ${package_src_dir}

#
# Copy build files
cp ./build.sh ${package_src_dir}
cp ./buildbin.${PACKAGE}*.sh ${package_src_dir}
# TODO: Generate package list instead copying

cat > ${package_src_dir}/package.list << EOF
# THIS FILE WAS AUTOGENERATED BY `basename $0`
#
${PACKAGE}|${build_version}|${apr_version}|${apu_version}|${ssl_version}|${svn_tagname}

EOF

# Copy other files
cp ../NMAKEcommon.inc $package_src_dir
cp ../../JBossORG-EULA.txt $package_src_dir
cp ../../srclib/apr/NMAKEmakefile $package_src_dir/srclib/apr-${apr_version}/
cp ../../srclib/apr-util/NMAKEmakefile $package_src_dir/srclib/apr-util-${apu_version}/

# Build the source tarball
buildtar ${build_working_dir} ${package_src_name} ${build_output_dir} ${package_src_name}

mkdir -p ${package_dist_dir}
mkdir -p ${package_output_dir}

# Now build the binaries
(cd ${package_src_dir}
  buildbin $build_output_dir
)
