#!/bin/sh
# Copyright(c) 2007 Red Hat Middleware, LLC,
# and individual contributors as indicated by the @authors tag.
# See the copyright.txt in the distribution for a
# full listing of individual contributors.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library in the file COPYING.LIB;
# if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# @author Jean-Frederic Clere
# @author Mladen Turk
#
echo ""
echo "Running : `basename $0` $LastChangedDate: 2007-07-18 03:18:06 -0400 (Wed, 18 Jul 2007) $"
echo ""
echo "Started : `date`"
echo "Common  : $1"
echo "Prefix  : $2"
echo "Output  : $3"
echo "OpenSSL : $4"
echo ""


# parameters
# $1: Location of the common libraries.
# $2: Destination location.
# $3: Location where to put the binaries.
# $4: Use OpenSSL.

common_loc=$1
prefix_loc=$2
output_loc=$3
has_openssl=$4

win_common_loc=`cygpath -w -a ${common_loc}`
win_prefix_loc=`cygpath -w -a ${prefix_loc}`

echo "Configuring tomcat-native"
add_conf="PREFIX=${win_prefix_loc}"
if $has_openssl ; then
  add_conf="${add_conf} WITH_OPENSSL=${win_common_loc}"
fi

mkdir -p ${output_loc}/bin/native
native_sources=srclib/`ls srclib | grep tomcat-native`
src_dir=`cygpath -w -a ${native_sources}`
(cd $native_sources
 nmake -f NMAKEmakefile WITH_APR=${win_common_loc} ${add_conf} SRCDIR=${src_dir} install
)

(cd service/jbosssch
 nmake -f NMAKEmakefile PREFIX=${win_prefix_loc} install
)

(cd service/jbosssvc
 nmake -f NMAKEmakefile PREFIX=${win_prefix_loc} install
)

(cd service/examples
 awk -f rewritesvc.awk service.bat ${output_loc}/bin/service.bat "4.2.1.GA" 4 2 "Windows ${BUILD_CPU}"
 cp README-service.txt ${output_loc}/bin
)

cp ${common_loc}/lib/libapr*.dll ${output_loc}/bin/native 2>/dev/null
if $has_openssl ; then
  cp ${common_loc}/bin/*eay32.dll ${output_loc}/bin/native 2>/dev/null
  cp ${common_loc}/bin/openssl.exe ${output_loc}/bin
fi
cp ${prefix_loc}/lib/*.dll ${output_loc}/bin/native 2>/dev/null
cp ${prefix_loc}/bin/jboss*.* ${output_loc}/bin 2>/dev/null

if [ -n "${CRT_REDIST}" ] ; then
  crt_redist_loc=`cygpath -u ${CRT_REDIST}`
  cp `cygpath -u ${CRT_REDIST}`/*.dll ${output_loc}/bin/native
fi
